library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity SUMVHDL is
    port (
        CLK: in STD_LOGIC;
        RST: in STD_LOGIC;
        D: in STD_LOGIC_VECTOR (3 downto 0);
        ST2: out STD_LOGIC;
        ST1: out STD_LOGIC;
        ST0: out STD_LOGIC;
        OV: out STD_LOGIC;
        U7A: out STD_LOGIC;
        U7B: out STD_LOGIC;
        U7C: out STD_LOGIC;
        U7D: out STD_LOGIC;
        U7E: out STD_LOGIC;
        U7F: out STD_LOGIC;
        U7G: out STD_LOGIC;
        U8A: out STD_LOGIC;
        U8B: out STD_LOGIC;
        U8C: out STD_LOGIC;
        U8D: out STD_LOGIC;
        U8E: out STD_LOGIC;
        U8F: out STD_LOGIC;
        U8G: out STD_LOGIC
    );
end SUMVHDL;

architecture SUMVHDL_arch of SUMVHDL is
   component HEX7SEG
       port (
          D: in STD_LOGIC_VECTOR (3 downto 0);
          EN: in STD_LOGIC;
          U: out STD_LOGIC_VECTOR (0 to 6)
       );
   end component;
    
   signal S: STD_LOGIC_VECTOR (2 downto 0);
   signal A, B: STD_LOGIC_VECTOR(7 downto 0);
   signal C: STD_LOGIC_VECTOR(8 downto 0);
   signal AH, AL, BH, BL, CH, CL, UHIN, ULIN: STD_LOGIC_VECTOR(3 downto 0);
   signal UHEN, ULEN: STD_LOGIC;
     
begin
   UH: HEX7SEG port map (D => UHIN, EN => UHEN,
                         U(0) => U7A,
                         U(1) => U7B,
                         U(2) => U7C,
                         U(3) => U7D,
                         U(4) => U7E,
                         U(5) => U7F,
                         U(6) => U7G );
   UL: HEX7SEG port map (D => ULIN, EN => ULEN,
                         U(0) => U8A,
                         U(1) => U8B,
                         U(2) => U8C,
                         U(3) => U8D,
                         U(4) => U8E,
                         U(5) => U8F,
                         U(6) => U8G );

   A(7 downto 4) <= AH;
   A(3 downto 0) <= AL;
   B(7 downto 4) <= BH;
   B(3 downto 0) <= BL;   
   C <= ('0' & A) + ('0' & B);
   CH <= C(7 downto 4);
   CL <= C(3 downto 0);
   
   UHIN <= CH when (S(2)='1') else
           D  when (S(0)='0') else
           AH when (S(1)='0') else
           BH;
   ULIN <= CL when (S(2)='1') else D;
   UHEN <= '1';
   ULEN <= S(2) or S(0);
   
   ST2 <= not S(2);
   ST1 <= not S(1);
   ST0 <= not S(0);
   OV <= not (S(2) and C(8));
   
   process (CLK, RST)
   begin
      if RST='0' then
         S <= "000";
         AH <= X"0";
         AL <= X"0";
         BH <= X"0";
         BL <= X"0";
      elsif FALLING_EDGE(CLK) then 
         if S(2)='0' then
            if S(1)='0' and S(0)='0' then
               AH <= D;
            elsif S(1)='0' and S(0)='1' then
               AL <= D;
            elsif S(1)='1' and S(0)='0' then
               BH <= D;
            else
               BL <= D;
            end if;
         end if;           
      
         case S is
            when "000"  => S <= "001";
            when "001"  => S <= "010";
            when "010"  => S <= "011";
            when "011"  => S <= "100";
            when others => S <= "000";
         end case;
      end if;
   end process;
   
end SUMVHDL_arch;

--configuration SUMVHDL_conf of SUMVHDL is
--   for SUMVHDL_arch
--      for all:HEX7SEG use entity
--         WORK.HEX7SEG(HEX7SEG_arch);
--      end for;
--   end for;
--end SUMVHDL_conf;
